# Preticanje

Postoji samo jedna traka, u jednosmjernoj cesti od Zračne luke **dr. Franje Tuđmana** do hotela. Dugačka je $L$ kilometara.

Tijekom IOI-ja 2007, $N + 1$ buseva prolazi tom cestom. Busevi su označeni od $0$ do $N$.
Bus $i$ ($0 \le i \lt N$) prema rasporedu napušta zračnu luku u $T[i]$-toj sekundi, i može prijeći jedan kilometar u $W[i]$ sekundi.
Bus $N$ je rezervni bus koji može prijeći $1$ kilometar u $X$ sekundi.
Vrijeme $Y$ kada on napušta zračnu luku još nije odlučeno.

Preticanje na cesti nije dozvoljeno, no busevima je dozvoljeno preticati se na posebni **stanicama**.
Postoji $M$ ($M \gt 1$) stanica, označenih od $0$ do $M - 1$, na različitim pozicijama na cesti.
Stanica $j$ ($0 \le j \lt M$) nalazi se $S[j]$ kilometara daleko od zračne luke.
Stanice su poredane uzlazno po udaljenosti od luke, točnije, $S[j] \lt S[j + 1]$ za svaki $0 \le j \le M - 2$.
Prva stanica je zračna luka, i zadnja stanica je hotel, točnije, $S[0] = 0$ i $S[M - 1] = L$.

Svaki bus putuje maksimalnom brzinom, osim ako ne stigne do sporijeg busa koji putuje ispred njega, u kojem slučaju u trenutku kad ga dostigne, prisiljen je putovati istom brzinom kao i sporiji bus, dokgod ne dođu do sljedeće stanice.
Stoga, brži busevi će uvijek preticati sporije buseve.

Točnije, za svaki $i$ te $j$ takav da $0 \le i \le N$ i $0 \le j \lt M$, vrijeme  $t_{i,j}$ (u sekundama) kada bus $i$ **stiže**
na stanicu $j$ određeno je na sljedeći način.
Neja je $t_{i, 0} = T[i]$ za svaki $0 \le i \lt N$, i neka je $t_{N, 0} = Y$.
Za svaki $j$ takav da je $0 \lt j \lt M$:
* Definiramo **očekivano vrijeme stizanja** (u sekundama) busa $i$ na stanicu $j$, označeno kao $e_{i, j},$ kao vrijeme
kada bi bus $i$ stigao na stanicu $j$ kada bi putovao punom brzinom od trenutka kada je stigao na stanicu $j-1$. Točnije, neka je
  - $e_{i,j} = t_{i,j-1} + W[i] \cdot (S[j]-S[j-1])$ za svaki $0 \le i \lt N$, i
  - $e_{N,j} = t_{N,j-1} + X \cdot (S[j]-S[j-1])$.
 * Bus $i$ stiže na stanicu $j$ u *maksimalnom* očekivanom vremenu stizanja među očekivanom vremenu stizanja busa $i$ i svih buseva koji su na stanicu $j-1$ stigli prije busa $i$. Točnije, neka je $t_{i, j}$ maksimum od vrijednosti $e_{i, j}$ i svih vrijednosti $e_{k, j}$ za koje je $0 \le k \le N$ te $t_{k,j-1} \lt t_{i,j-1}$.

Gospodin Malnar moli Vas da izračunate vrijeme stizanja rezervnog busa (busa $N$).
Vaš je zadatak odgovoriti na $Q$ Malnarovih upita, koji su sljedećeg oblika: kada bi rezervni bus krenuo sa zračne luke nakon $Y$ sekundi, kada bi stigao u hotel?

## Implementacijski detalji

Vaš je zadatak implementirati sljedeće funkcije.

```
void init(int L, int N, int64[] T, int[] W, int X, int M, int[] S)
```

* $L$: duljina ceste
* $N$: broj buseva koji nisu rezervni
* $T$: niz duljine $N$ koji opisuje vremena u kojima busevi koji nisu rezervni kreću sa zračne luke.
* $W$: niz duljine $N$ koji opisuje maksimalne brzine u kojima busevi koji nisu rezervni kreću sa zračne luke.
* $X$: vrijreme koje je potrebno rezervnom busu kako bi prešao $1$ kilometar
* $M$: broj stanica
* $S$: niz duljine $M$ koji opisuje udaljenosti stanica od zračne luke.
* Ova funkcija pozvana je točno jednom prije svih poziva funkcije `arrival_time`.

```
int64 arrival_time(int64 Y)
```

* $Y$: vrijeme kada rezervni bus (bus $N$) treba krenuti sa zračne luke
* Ova funkcija trebala bi vratiti vrijeme kad bi rezervni bus stigao u hotel.
* Ova funkcija bit će pozvana točno $Q$ puta.

## Primjer

Promatrajte sljedeći niz poziva funkcija.

```
init(6, 4, [20, 10, 40, 0], [5, 20, 20, 30], 10, 4, [0, 1, 3, 6])
```

Zanemarivši bus $4$ (čije vrijeme polaska još nije određeno), sljedeća tablica prikazuje očekivana vremena stizanja i stvarna vremena stizanja buseva koji nisu rezervni na svaku stanicu.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$

Vremena stizanja na stanicu $0$ su vremena u kojima busevi polaze sa zračne luke. Točnije, $t_{i, 0} = T[i]$ za $0 \le i \le 3$.

Očekivana i stvarna vremena stizanja na stanicu $1$ mogu se izračunati na sljedeći način:
* Očekivana vremena stizanja na stanicu $1$:
  - Bus $0$: $e_{0,1} = t_{0,0} + W[0] \cdot (S[1]-S[0]) = 20 + 5 \cdot 1 = 25$.
  - Bus $1$: $e_{1,1} = t_{1,0} + W[1] \cdot (S[1]-S[0]) = 10 + 20 \cdot 1 = 30$.
  - Bus $2$: $e_{2,1} = t_{2,0} + W[2] \cdot (S[1]-S[0]) = 40 + 20 \cdot 1 = 60$.
  - Bus $3$: $e_{3,1} = t_{3,0} + W[3] \cdot (S[1]-S[0]) = 0 + 30 \cdot 1 = 30$.
* Vremena stizanja na stanicu $1$:
  - Busevi $1$ i $3$ stižu na stanicu $0$ prije busa $0$, stoga $t_{0,1} = \max([e_{0,1},e_{1,1},e_{3,1}]) = 30$.
  - Bus $3$ stiže na stanicu $0$ prije busa $1$, stoga $t_{1,1} = \max([e_{1,1},e_{3,1}]) = 30$.
  - Bus $0$, bus $1$ i bus $3$ stižu na stanicu $0$ prije busa $2$, stoga $t_{2,1} = \max([e_{0,1},e_{1,1},e_{2,1},e_{3,1}]) = 60$.
  - Nijedan bus ne stiže na stanicu $0$ prije busa $3$, stoga $t_{3,1} = \max([e_{3,1}]) = 30$.

```
arrival_time(0)
```

Busu $4$ treba $10$ sekundi kako bi prešao $1$ kilometar, i određeno je da kreće sa zračne luke u $0$-toj sekundi.
U tom slučaju, sljedeća tablica prikazuje vrijeme stizanja za svaki bus.
Jedina promjena s obzirom na očekivano i stvarno vrijeme stizanja buseva koji nisu rezervni je podcrtana.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:-----------------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $\underline{60}$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $0$       || $10$      | $10$      || $30$      | $30$      || $60$      | $60$

Vidimo da bus $4$ stiže u hotel u $60$-toj sekundi.
Stoga, funkcija treba vratiti $60$.

```
arrival_time(50)
```

Bus $4$ sada treba kreunti sa zračne luke u $50$-toj sekundi.
U tom slučaju, nema promjena u vremenima stizanja za buseve koji nisu rezervni u usporedbi s početnom tablicom.
Vremena stizanja prikazana su u sljedećoj tablici.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $50$      || $60$      | $60$      || $80$      | $90$      || $120$     | $130$

Bus $4$ pretiče sporiji bus $2$ na stanici $1$ jer stižu u isto vrijeme.
Sljedeće, bus $4$ dostiže bus $3$ između stanice $1$ i stanice $2$, stoga bus $4$ stiže na stanicu $2$ u $90$-toj sekundi umjesto $80$-te.
Nakon napuštanja stanice $2$, bus $4$ stiže bus $1$ dok ne stigne u hotel.
Bus $$ stiže u hotel u $130$-toj sekundi.
Stoga, funkcija bi trebala vratiti $130$.

Možemo prikazati vrijeme potrebno svakom busu kako bi stigao do određene udaljenosti od zračne luke.
Neka x-os prikazuje udaljenost od zračne luke (u kilometrima) i y-os prikazuje vrijeme ( u sekundama).
Okomite iscrtkane linije označavajnu pozicije stanica.
Popunjene vodoravne linije ( zajedno s oznakama buseva ) prikazuju buseve koji nisu rezervni.
Istočkana crna linija prikazuje rezervni bus.

 `arrival_time(0)`               | `arrival_time(50)`
:-------------------------------:|:------------------------------:
![](overtaking-query1-new.png)   | ![](overtaking-query2-new.png)

## Ograničenja

* $1 \le L \le 10^9$
* $1 \le N \le 1\,000$
* $0 \le T[i] \le 10^{18}$ (za svaki $i$ takav da $0 \le i \lt N$)
* $1 \le W[i] \le 10^9$ (za svaki $i$ takav da $0 \le i \lt N$)
* $1 \le X \le 10^9$
* $2 \le M \le 1\,000$
* $0 = S[0] \lt S[1] \lt \cdots \lt S[M-1] = L$
* $1 \le Q \le 10^6$
* $0 \le Y \le  10^{18}$

## Podzadatci

1. (9 bodova) $N = 1, Q \le 1\,000$
1. (10 bodova) $M = 2, Q \le 1\,000$
1. (20 bodova) $N, M, Q \le 100$
1. (26 bodova) $Q \le 5\,000$
1. (35 bodova) Nema dodatnih ograničenja

## Probni ocjenjivač

Probni ocjenjivač učitava ulaz u sljedećem obliku:

* $1.$ redak: $L \; N \; X \; M \; Q$
* $2.$ redak: $T[0] \; T[1] \; \ldots \; T[N-1]$
* $3.$ redak: $W[0] \; W[1] \; \ldots \; W[N-1]$
* $4.$ redak: $S[0] \; S[1] \; \ldots \; S[M-1]$
* $(5 + k).$ redak ($0 \le k \lt Q$): $Y$ za svaki upit $k$

Probni ocjenjivač ispisuje Vaše odgovore u sljedećem obliku.

* $(1 + k).$ redak ($0 \le k \lt Q$): vrijednost koju je funkcija `arrival_time` vratila pri upitu $k$
