# Overtaking

Postoji autoput od Podgorice do Virpazara, koji je samo u jednom smjeru (od Podgorice prema Virpazaru), i takođe trenutno radi samo jedna traka. Zbog nemogućnosti lakog preticanja, takozvanog preticanja bez gasiranja, nazovimo ga Gasput. Gasput je dugačak $L$ kilometara.

Za vrijeme Dana vina i ukljeve 2023, $N+1$ autobusa prelaze ovim putem. Autobusi su numerisani od $0$ to $N$.
Autobus $i$ ($0 \le i \lt N$) je zakazan da napusti Podgoricu u $T[i]$-oj od početka događaja, i može da pređe $1$ kilometar u $W[i]$ sekundi
Autobus $N$ je zamjena i može da pređe $1$ kilometar u $X$ sekundi.
Vrijeme $Y$ kada će krenuti iz Podgorice nije još odlučeno.

Preticanje je nedozvoljeno na putu, ali su autobusi omogućeni da se pretiču u **gas stanicama**.
Postoji $M > 1$ gas stanica na gasputu, koje su numerisane od $0$ do $M - 1$, na različitim pozicijama na gasputu.
Gas stanica $j$ ($0 \le j \lt M$) se nalazi na $S[j]$ kilometara od Podgorice, na gasputu. Gas stanice su sortirane rastuće po rastojanju od Podgorice, to jest, $S[j] \lt S[j+1]$ za svako $0 \le j \le M - 2$.
Prva gas stanica je Podgorica, i posljednja gas stanica je Virpazar, to jest, $S[0] = 0$ i $S[M-1] = L$.

Svaki autobus putuje maksimalnom brzinom, osim ako se susretne sa sporijim autobusom koji ide putem ispred njega, u kom slučaju se oni grupišu i moraju da idu istom manjom brzinom, dok ne dođu do sledeće gas stanice. Tamo, brži autobusi će se izgasirati i prestići će sporije.

Formalno, za svako $i$ i $j$ tako da $0 \le i \le N$ i $0 \le j \lt M$, vrijeme $t_{i,j}$ (u sekundama) kada autobus $i$ **stiže u** gas stanicu $j$ je definisan kao.
Ako $j = 0$, neka je onda $t_{i,0} = T[i]$ za svako $i \lt N$, i neka je $t_{N,0} = Y$.
Inače, za svako $j$ takvo da je $0 \lt j \lt M$:
* Definišimo *očekivano vrijeme stizanja* autobusa $i$ u gas stanicu $j$ kao vrijeme kada bi autobus $i$ stigao u gas stanicu $j$ kada bi putovao punom brzinom od vremena kada je stigao u gas stanicu $j-1$. Neka je
  - $e_{i,j} = t_{i,j-1} + W[i] \cdot (S[j]-S[j-1])$ za svako $i \lt N$, i 
  - $e_{N,j} = t_{N,j-1} + X \cdot (S[j]-S[j-1])$.
* Autobus $i$ stiže u gas stanicu $j$ u *maksimumu* očekivanih vremena dolazaka autobusa $i$ i bilo kog drugog autobusa koji je stigao u stanicu $j-1$ prije autobusa $i$. Formalno, neka je $t_{i,j}$ maksimum $e_{i,j}$ i svakog $e_{k,j}$ za koje $0 \le k \le N$ i $t_{k,j-1} \lt t_{i,j-1}$.

Organizatori Dana vina i ukljeve žele da zakažu polazak rezervnog autobusa.
Vaš zadatak je da odgovorite na $Q$ pitanja organizatora, koja su sledećeg oblika: za dato vrijeme $Y$ (u sekundama) kada bi autobus $N$ napustio Podgoricu, u koje vrijeme bi stigao u Virpazar?

## Detalji implementacije

Vaš zadatak je da implementirate sledeću funkciju.

```
void init(int L, int N, int64[] T, int[] W, int X, int M, int[] S)
```

* $L$: dužina gasputa.
* $N$: broj zakazanih autobusa.
* $T$: niz dužine $N$ koji predstavlja vremena u kojima autobusi $0,\ldots,N-1$ kreću iz Podgorice.
* $W$: niz dužine $N$ koji predstavlja maksimalne brzine autobusa $0,\ldots,N-1$.
* $X$: vrijeme koje treba rezervnom autobusu da pređe $1$ kilometar.
* $M$: broj gas stanica.
* $S$: niz dužine $M$ koji predstavlja rastojanja gas stanica od Podgorice.
* Ova funkcija se poziva tačno jednom za svaki test primjer, prije bilo kog poziva ka `arrival_time`.

```
int64 arrival_time(int64 Y)
```

* $Y$: vrijeme kada je predviđeno da rezervni autobus krene iz Podgorice.
* Funkcija treba da vrati vrijeme u koje bi autobus $N$ stigao u Virpazar.
* Ova funkcija će biti pozvana tačno $Q$ puta.

## Primjer

Posmatrajmo sledeće pozive funkcija:

```
init(6, 4, [20, 10, 40, 0], [5, 20, 20, 30], 10, 4, [0, 1, 3, 6])
```

Ignorišući autobus $4$ (koji još nije zakazan), sledeća tabela prikazuje očekivana i prava vremena dolazaka autobusa $0,1,2$ i $3$ u svaku od gas stanica:

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$

Vremena dolazaka u gas stanicu $0$ su vremena u kojima su autobusi zakazani da napuste aerodrom. To jest, $t_{i,0} = T[i]$ za $i=0,1,2$ i $3$.

Očekivana i prava vremena dolazaka u gas stanicu $1$ su izračunata na sledeći način:
* Očekivana vremena dolaska u gas stanicu $1$:
  - Autobus $0$: $e_{0,1} = t_{0,0} + W[0] \cdot (S[1]-S[0]) = 20 + 5 \cdot 1 = 25$.
  - Autobus $1$: $e_{1,1} = t_{1,0} + W[1] \cdot (S[1]-S[0]) = 10 + 20 \cdot 1 = 30$.
  - Autobus $2$: $e_{2,1} = t_{2,0} + W[2] \cdot (S[1]-S[0]) = 40 + 20 \cdot 1 = 60$.
  - Autobus $3$: $e_{3,1} = t_{3,0} + W[3] \cdot (S[1]-S[0]) = 0 + 30 \cdot 1 = 30$.
* Vremena dolazaka u gas stanicu $1$:
  - Autobusi $1$ i $3$ stižu u gas stanicu $0$ ranije od autobusa $0$, pa je $t_{0,1} = \max(e_{0,1},e_{1,1},e_{3,1}) = 30$.
  - Autobus $3$ stiže u gas stanicu $0$ ranije od autobusa $1$, pa je $t_{1,1} = \max(e_{1,1},e_{3,1}) = 30$.
  - Autobusi $0$, $1$ i $3$ stižu u gas stanicu $0$ ranije od autobusa $2$, pa je $t_{2,1} = \max(e_{0,1},e_{1,1},e_{2,1},e_{3,1}) = 60$.
  - Nijedan autobus ne stiže u gas stanicu $0$ prije autobusa $3$, te je $t_{3,1} = \max(e_{3,1}) = 30$.


```
arrival_time(0)
```

Autobusu $4$ treba $10$ sekundi da otputuje $1$ kilometar i sada je zakazan da napusti Podgoricu u $0$-oj sekundi
U ovom slučaju, sledeća tabela prikazuje vremena dolazaka svakog autobusa.
Razlike u odnosu na inicijalnu tabelu su podvučene.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:-----------------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $\underline{60}$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $0$       || $10$      | $10$      || $30$      | $30$      || $60$      | $60$

Vidimo da autobus $4$ stiže u Virpazar u $60$-oj sekundi.
Dakle, procedura treba da vrati $60$.

```
arrival_time(50)
```

Autobus $4$ je sada zakazan da krene iz Podgorice u $50$-oj sekundi.
U ovom slučaju, nema promjena dolazaka autobusa $0$, $1$, $2$ i $3$.
Vremena dolazaka su prikazana u sledećoj tabeli.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $50$      || $60$      | $60$      || $80$      | $90$      || $120$     | $130$

Autobus $4$ pretiče sporiji autobus $2$ u gas stanici $1$ jer stižu u isto vrijeme.
Sledeće, autobus $4$ biva grupisan sa autobusom $3$ između gas stanica $1$ i $2$, što povlači da autobus $4$ stiže u gas stanicu $2$ u $90$-oj sekundi umesto u $80$-oj.
Nakon odlaska iz gas stanice $2$, autobus $4$ se grupiše sa autobusom $1$ dok ne stignu u Virpazar.
Autobus $4$ stiže u Virpazar u $130$-oj sekundi.
Dakle, procedura treba da vrati $130$.

Možemo iscrtati vrijeme koje je potrebno svakom autobusu da stigne na svaku udaljenost od Podgorice. X-os dijagrama predstavlja udaljenost od Podgorice (u kilometrima), a y-osa grafikona predstavlja vrijeme (u sekundama). Vertikalne tačkaste linije označavaju položaje gas stanica. Različite pune linije (propraćene indeksima autobusa) predstavljaju četiri nerezervna autobusa. Isprekidana crna linija predstavlja rezervni autobus.


 `arrival_time(0)`           | `arrival_time(50)`
:---------------------------:|:---------------------------:
![](overtaking-query1.png)   | ![](overtaking-query2.png)

## Ograničenja

* $1 \le L \le 10^9$
* $1 \le N \le 1\,000$
* $0 \le T[i] \le 10^{18}$ (za svako $i$ tako da je $0 \le i \lt N$)
* $1 \le W[i] \le 10^9$ (za svako $i$ tako da je $0 \le i \lt N$)
* $1 \le X \le 10^9$
* $2 \le M \le 1\,000$
* $0 = S[0] \lt S[1] \lt \cdots \lt S[M-1] = L$
* $1 \le Q \le 10^6$
* $0 \le Y \le  10^{18}$

## Podzadaci

1. (9 poena) $N = 1, Q \le 1\,000$
1. (10 poena) $M = 2, Q \le 1\,000$
1. (20 poena) $N, M, Q \le 100$
1. (26 poena) $Q \le 5\,000$
1. (35 poena) Bez dodatnih ograničenja.

## Sample grader

Sample grader očekuje ulaz u sledećem formatu:

* linija $1$: $L \; N \; X \; M \; Q$
* linija $2$: $T[0] \; T[1] \; \ldots \; T[N-1]$
* linija $3$: $W[0] \; W[1] \; \ldots \; W[N-1]$
* linija $4$: $S[0] \; S[1] \; \ldots \; S[M-1]$
* linija $5 + k$ ($0 \le k \lt Q$): $Y$ za pitanje $k$

Grader ispisuje vaše odgovore u sledećem formatu:

* linija $1 + k$ ($0 \le k \lt Q$): povratna vrijednost funkcije `arrival_time` za pitanje $k$
