# Гасирање

Постоји аутопут од Београда до Лесковца, који је само у једном смеру (од Београда према Лесковцу), и такође тренутно ради само једна трака. Због немогућности лаког претицања, такозваног претицања без гасирања, назовимо га Гаспут. Гаспут је дугачак $L$ километара.

За време Роштиљијаде у Лесковцу 2023, $N+1$ аутобуса пролазе овим путем. Аутобуси су означени од $0$ to $N$.
Аутобус $i$ ($0 \le i \lt N$) је заказан да напусти Београд у $T[i]$-ој од почетка догађаја, и може д пређе $1$ километар у $W[i]$ секунди
Аутобус $N$ је резервни аутобус и може да пређе $1$ километар у $X$ секунди.
Време $Y$ када ће кренути из Београда није још одлучено.

Претицање је недозвољено на путу, али су аутобуси омогућени да се претичу у **гас станицама**.
Постоји $M > 1$ гас станица на гаспуту, које су нумерисане од $0$ до $M - 1$, на различитим позицијама на гаспуту.
Гас станица $j$ ($0 \le j \lt M$) се налази на $S[j]$ километара од Београда, на гаспуту. Гас станице су сортиране растуће по растојању од Београда, то јест, $S[j] \lt S[j+1]$ за свако $0 \le j \le M - 2$.
Прва гас станица је Београд, и последња гас станица је Лесковац, то јест, $S[0] = 0$ и $S[M-1] = L$.

Сваки аутобус путује максималном брзином, осим ако се сусретне са споријим аутобусом који иде путем испред њега, у ком случају се они групишу и морају да иду истом мањом брзином, док не дођу до следеће гас станице. Тамо, бржи аутобуси ће се изгасирати и престићи ће спорије.

Формално, за свако $i$ и $j$ тако да $0 \le i \le N$ и $0 \le j \lt M$, време $t_{i,j}$ (у секундама) када аутобус $i$ **стиже у** гас станицу $j$ је дефинисан као.
Ако $j = 0$, нека је онда $t_{i,0} = T[i]$ за свако $i \lt N$, и нека је $t_{N,0} = Y$.
Иначе, за свако $j$ такво да је $0 \lt j \lt M$:
* Дефинишимо *очекивано време стизања* аутобуса $i$ у гас станицу $j$ као време када би аутобус $i$ стигао у гас станицу $j$ када би путовао пуном брзином од времена када је стигао у гас станицу $j-1$. Нека је
  - $e_{i,j} = t_{i,j-1} + W[i] \cdot (S[j]-S[j-1])$ за свако $i \lt N$, и 
  - $e_{N,j} = t_{N,j-1} + X \cdot (S[j]-S[j-1])$.
* Аутобус $i$ стиже у гас станицу $j$ у *максимуму* очекиваних времена долазака аутобуса $i$ и било ког другог утобуса који је стигао у станицу $j-1$ пре аутобуса $i$. Формално, нека је $t_{i,j}$ максимум $e_{i,j}$ и сваког $e_{k,j}$ за које $0 \le k \le N$ и $t_{k,j-1} \lt t_{i,j-1}$.

Организатори Роштиљијаде желе да закажу полазак резервног аутобуса.
Ваш задатак је да одговорите на $Q$ питања организатора, која су следећег облика: за дато време $Y$ (у секундама) када би аутобус $N$ (резервни аутобус) напустио Београд, у које време би стигао у Лесковац?

## Детаљи имплементације

Ваш задатак је да имплементирате следећу процедуру.

```
void init(int L, int N, int64[] T, int[] W, int X, int M, int[] S)
```

* $L$: дужина гаспута.
* $N$: број заказаних аутобуса.
* $T$: низ дужине $N$ који представља времена у којима аутобуси $0,\ldots,N-1$ крећу из Београда.
* $W$: низ дужине $N$ који представља максималне брзине аутобуса $0,\ldots,N-1$.
* $X$: време које треба резервном аутобусу да пређе $1$ километар.
* $M$: број гас станица.
* $S$: низ дужине $M$ који представља растојања гас станица од Београда.
* Ова процедура се позива тачно једном за сваки тест пример, пре било ког позива ка `arrival_time`.

```
int64 arrival_time(int64 Y)
```

* $Y$: време када је предвиђено да резервни аутобус (аутобус $N$)  крене из Београда.
* Процедура треба да врати време у које би аутобус $N$ стигао у Лесковац.
* Ова процедура ће бити позвана тачно $Q$ пута.

## Пример

Посматрајмо следеће позиве функција:

```
init(6, 4, [20, 10, 40, 0], [5, 20, 20, 30], 10, 4, [0, 1, 3, 6])
```

Игноришући аутобус $4$ (који још није заказан), следећа табела приказује очекивана и права времена долазака аутобуса $0,1,2$ и $3$ у сваку од гас станица:

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$

Времена долазака у гас станицз $0$ су времена у којима су аутобуси заказани да напусте аеродром. То јест, $t_{i,0} = T[i]$ за $i=0,1,2$ и $3$.

Очекивана и права времена долазака у гас станицу $1$ су израчуната на следећи начин:
* Очекивана времена доласка у гас станицу $1$:
  - Аутобус $0$: $e_{0,1} = t_{0,0} + W[0] \cdot (S[1]-S[0]) = 20 + 5 \cdot 1 = 25$.
  - Аутобус $1$: $e_{1,1} = t_{1,0} + W[1] \cdot (S[1]-S[0]) = 10 + 20 \cdot 1 = 30$.
  - Аутобус $2$: $e_{2,1} = t_{2,0} + W[2] \cdot (S[1]-S[0]) = 40 + 20 \cdot 1 = 60$.
  - Аутобус $3$: $e_{3,1} = t_{3,0} + W[3] \cdot (S[1]-S[0]) = 0 + 30 \cdot 1 = 30$.
* Времена долазака у гас станицу $1$:
  - Аутобуси $1$ и $3$ стиже у гас станицу $0$ раније од аутобуса $0$, па је $t_{0,1} = \max(e_{0,1},e_{1,1},e_{3,1}) = 30$.
  - Аутобус $3$ стиже у гас станицу $0$ раније од аутобуса $1$, па је $t_{1,1} = \max(e_{1,1},e_{3,1}) = 30$.
  - Аутобуси $0$, $1$ и $3$ стижу у гас станицу $0$ раније од аутобуса $2$, па је $t_{2,1} = \max(e_{0,1},e_{1,1},e_{2,1},e_{3,1}) = 60$.
  - Ниједан аутобус не стиже у гас станицу $0$ пре аутобуса $3$, те је $t_{3,1} = \max(e_{3,1}) = 30$.

```
arrival_time(0)
```

Аутобусу $4$ треба $10$ секунди да отпутује $1$ километар и сада је заказан да напусти Београд у $0$-ој секунди
У овом случају, следећа табела приказује времена долазака сваког аутобуса.
Разлике у односу на иницијалну табелу су подвучене.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:-----------------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $\underline{60}$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $0$       || $10$      | $10$      || $30$      | $30$      || $60$      | $60$

Видимо да аутобус $4$ стиже у Лесковац у $60$-ој секунди.
Дакле, процедура треба да врати $60$.

```
arrival_time(50)
```

Аутобус $4$ је сада заказан да крене из Београда у $50$-ој секунди.
У овом случају, нема промена долазака аутобуса $0$, $1$, $2$ и $3$.
Времена долазака су приказана у следећој табели.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $50$      || $60$      | $60$      || $80$      | $90$      || $120$     | $130$

Аутобус $4$ претиче спорији аутобус $2$ у гас станици $1$ јер стижу у исто време.
Следеће, аутобус $4$ бива груписан са аутобусом $3$ између гас станица $1$ и $2$, што повлачи да аутобус $4$ стиже у гас станицу $2$ у $90$-ој секунди уместо у $80$-ој.
Након одласка из гас станице $2$, аутобус $4$ се групише са аутобусом $1$ док не стигну у Лесковац.
Аутобус $4$ стиже у Лесковац у $130$-ој секунди.
Дакле, процедура треба да врати $130$.

На следећинм графицима, приказујемо растојања аутобуса од Београда кроз време. Тачкаста црна линиј представља резервни аутобус.

 `arrival_time(0)`           | `arrival_time(50)`
:---------------------------:|:---------------------------:
![](overtaking-query1.png)   | ![](overtaking-query2.png)

## Ограничења

* $1 \le L \le 10^9$
* $1 \le N \le 1\,000$
* $0 \le T[i] \le 10^{18}$ (за свако $i$ тако да је $0 \le i \lt N$)
* $1 \le W[i] \le 10^9$ (за свако $i$ тако да је $0 \le i \lt N$)
* $1 \le X \le 10^9$
* $2 \le M \le 1\,000$
* $0 = S[0] \lt S[1] \lt \cdots \lt S[M-1] = L$
* $1 \le Q \le 10^6$
* $0 \le Y \le  10^{18}$

## Подзадаци

1. (9 поена) $N = 1, Q \le 1\,000$
1. (10 поена) $M = 2, Q \le 1\,000$
1. (20 поена) $N, M, Q \le 100$
1. (26 поена) $Q \le 5\,000$
1. (35 поена) Без додатних ограничења.

## Пример оцењивача (sample grader)

Оцењивач учитава улаз у следећем формату:

* линија $1$: $L \; N \; X \; M \; Q$
* линија $2$: $T[0] \; T[1] \; \ldots \; T[N-1]$
* линија $3$: $W[0] \; W[1] \; \ldots \; W[N-1]$
* линија $4$: $S[0] \; S[1] \; \ldots \; S[M-1]$
* линија $5 + k$ ($0 \le k \lt Q$): $Y$ за питање $k$

Оцењивач исписује ваше одговоре у следећем формату:

* линија $1 + k$ ($0 \le k \lt Q$): повратна вредност процедуре `arrival_time` за питање $k$
