# Predchádzanie

Z letiska v Budapešti do hotela Forrás vedie jednosmerná jednopruhová cesta. Táto cesta má dĺžku $L$ kilometrov.

Počas príchodu tímov na súťaž IOI 2023 prejde touto cestou $N+1$ autobusov, ktoré sú očíslované od $0$ do $N$.
Autobus $i$ ($0 \le i \lt N$) má naplánovaný odchod z letiska v čase $T[i]$ sekúnd (od začiatku podujatia) a dokáže prejsť $1$ kilometer za $W[i]$ sekúnd.
Autobus $N$ je rezervný a dokáže prejsť $1$ kilometer za $X$ sekúnd.
Čas $Y$, kedy odíde z letiska ešte nebol určený.

Predchádzanie nie je povolené nikde na celej trase, ale nachádza sa popri nej $M$ ($M > 1$ ) odpočívadiel, kde môžu autobusy zastaviť. Tieto odpočívadlá sú očíslované od $0$ do $M - 1$, pričom sa nachádzajú na rôznych miestach pozdĺž cesty. Odpočívadlo $j$ ($0 \le j \lt M$) sa nachádza vo vzdialenosti $S[j]$ kilometrov od letiska. Odpočívadlá sú zoradené podľa rastúcej vzdialenosti od letiska, teda  $S[j] \lt S[j+1]$ pre každé $0 \le j \le M - 2$. Prvé odpočívadlo je na letisku a posledné pri hoteli, teda $S[0] = 0$ and $S[M-1] = L$.

Každý autobus ide svojou maximálnou rýchlosťou, až kým nedostihne pred ním pomalšie idúci autobus. Vtedy spomalí na rýchlosť pomalšieho autobusu a tou ide až po najbližšie odpočívadlo. Tam pomalší autobus na zanedbateľne krátky okamih zastaví a pustí rýchlejší pred seba. Pri odpočívadle takto vždy rýchlejšie autobusy predbehnú pomalšie.

Formálne, pre každé $i$ a $j$ také, že $0 \le i \le N$ a $0 \le j \lt M$, čas $t_{i,j}$ (v sekundách), kedy autobus $i$ **príde** na odpočívadlo $j$ je definovaný nasledovne:

Pre $j = 0$ zjavne platí $t_{i,0} = T[i]$ pre každé $i \lt N$, a navyše $t_{N,0} = Y$.

Potom postupne pre každé $j$ také, že $0 \lt j \lt M$, určíme tieto časy v dvoch krokoch: najskôr vypočítame očakávané časy príchodu a potom z nich tie skutočné. Presnejšie:
* Definujme *očakávaný čas príchodu* autobusu $i$ na odpočívadlo $j$ ako čas, kedy by autobus $i$ prišiel na odpočívadlo $j$, ak by z odpočívadla $j-1$ išiel celý čas svojou maximálnou rýchlosťou. To znamená, že 
  - $e_{i,j} = t_{i,j-1} + W[i] \cdot (S[j]-S[j-1])$ pre každé $i \lt N$ a 
  - $e_{N,j} = t_{N,j-1} + X \cdot (S[j]-S[j-1])$.
* Autobus $i$ skutočne príde na odpočívadlo $j$ v čase rovnom *maximu* z očakávaného času príchodu autobusu $i$ a všetkých očakávaných časov príchodu autobusov, ktoré prišli na odpočívadlo $j-1$ ostro skôr ako autobus $i$. Teda $t_{i,j}$ je maximum z $e_{i,j}$ a všetkých $e_{k,j}$ pre tie $0 \le k \le N$, pre ktoré $t_{k,j-1} \lt t_{i,j-1}$.

Organizátori IOI chcú naplánovať čas odchodu rezervného autobusu (autobusu $N$). Vašou úlohou je zodpovedať $Q$ otázok organizátorov. Každá otázka je nasledovného formátu: Pre daný čas $Y$ (v sekundách) odchodu rezervného autobusu $N$ vypočítajte čas príchodu rezervného autobusu do hotela.

## Implementačné detaily

Vašou úlohou je implementovať nasledujúce funkcie:

```
void init(int L, int N, int64[] T, int[] W, int X, int M, int[] S)
```

* $L$: dĺžka cesty
* $N$: počet autobusov (bez rezervného)
* $T$: pole dĺžky $N$ popisujúce časy plánovaných odchodov autobusov $0,\ldots,N-1$ z letiska
* $W$: pole dĺžky $N$ popisujúce, za aký čas prejdú kilometer autobusy $0,\ldots,N-1$.
* $X$: čas potrebný na prejdenie jedného kilometra pre rezervný autobus.
* $M$: počet odpočívadiel
* $S$: pole dĺžky $M$ popisujúce vzdialenosti odpočívadiel od letiska.
* Táto funkcia je zavolaná práve raz pre každú testovaciu sadu, vždy pred prvým volaním funkcie `arrival_time`.

```
int64 arrival_time(int64 Y)
```

* $Y$: čas, kedy je plánovaný odchod rezervného autobusu (autobusu $N$) z letiska
* Táto funkcia má vrátiť čas, kedy rezervný autobus $N$ príde do hotela.
* Táto funkcia bude zavolaná presne $Q$-krát.

## Príklad

Uvažujme nasledovnú postupnosť volaní:

```
init(6, 4, [20, 10, 40, 0], [5, 20, 20, 30], 10, 4, [0, 1, 3, 6])
```

Ignorujúc autobus $4$ (ktorý ešte nebol naplánovaný), nasledujúca tabuľka uvádza očakávané ($e$) a skutočné ($t$) časy príchodu autobusov $0,1,2$ a $3$ na každé odpočívadlo:

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$

Časy na odpočívadle $0$ sú časy plánovaného odchodu z letiska. Teda, $t_{i,0} = T[i]$ pre $i=0,1,2$ a $3$.

Očakávané a skutočné časy príchodu na odpočívadlo $1$ sú vypočítané nasledovne:
* Očakávané časy príchodu na odpočívadlo $1$:
  - Autobus $0$: $e_{0,1} = t_{0,0} + W[0] \cdot (S[1]-S[0]) = 20 + 5 \cdot 1 = 25$.
  - Autobus $1$: $e_{1,1} = t_{1,0} + W[1] \cdot (S[1]-S[0]) = 10 + 20 \cdot 1 = 30$.
  - Autobus $2$: $e_{2,1} = t_{2,0} + W[2] \cdot (S[1]-S[0]) = 40 + 20 \cdot 1 = 60$.
  - Autobus $3$: $e_{3,1} = t_{3,0} + W[3] \cdot (S[1]-S[0]) = 0 + 30 \cdot 1 = 30$.
* Časy príchodu na odpočívadlo $1$:
  - Autobusy $1$ a $3$ prídu na odpočívadlo $0$ skôr ako autobus $0$, teda $t_{0,1} = \max(e_{0,1},e_{1,1},e_{3,1}) = 30$.
  - Autobus $3$ príde na stanicu $0$ skôr ako autobus $1$, teda $t_{1,1} = \max(e_{1,1},e_{3,1}) = 30$.
  - Autobusy $0$, $1$ a $3$ prídu na odpočívadlo $0$ skôr ako autobus $2$, teda $t_{2,1} = \max(e_{0,1},e_{1,1},e_{2,1},e_{3,1}) = 60$.
  - Žiaden autobus nepríde na odpočívadlo $0$ skôr ako autobus $3$, teda $t_{3,1} = \max(e_{3,1}) = 30$.

```
arrival_time(0)
```

Autobusu $4$ trvá prejsť $1$ kilometer $10$ sekúnd a je naplánovaný odísť z letiska v čase $0$.
V tomto prípade sú jednotlivé očakávané aj skutočné časy príchodu na jednotlivých odpočívadlách uvedené nižšie, pričom zmeny oproti predchádzajúcej tabuľke sú zvýraznené podčiarknutím.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:-----------------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $\underline{60}$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $0$       || $10$      | $10$      || $30$      | $30$      || $60$      | $60$

Vidíme, že autobus $4$ dorazí do hotela v $60$-tej sekunde. Teda správna návratová hodnota funkcie je $60$.

```
arrival_time(50)
```

Autobus $4$ má teraz naplánovaný odchod v $50$-tej sekunde. V tomto prípade nedochádza k žiadnej zmene príchodov autobusov $0$, $1$, $2$ a $3$ (viď tabuľka nižšie).

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $50$      || $60$      | $60$      || $80$      | $90$      || $120$     | $130$

Autobus $4$ obehne pomalší autobus $2$ na odpočívadle $1$, kde prichádzajú v rovnaký čas.
Neskôr je autobus $4$ spomaľovaný autobusom $3$ medzi odpočívadlami $1$ a $2$, čo spôsobí príchod autobusu $4$ na odpočívadlo $2$ až v čase $90$ namiesto $80$ (sekúnd).
Po opustení odpočívadla $2$ autobus $4$ stretne autobus $1$ a tým bude spomaľovaný až do príchodu k hotelu.
Autobus $4$ príde k hotelu v čase $130$ sekúnd. Teda správna hodnotu, ktorú má vaša funkcia vrátiť, je $130$.

Ku obom príkladom si môžeme nakresliť graf času príchodu v závislosti od vzdialenosti od letiska pre každý autobus.
Na osi x je uvedená vzdialenosť od letiska (v kilometroch) a na osi y je uvedený čas (v sekundách). Zvislé prerušované čiary označujú polohy odpočívadiel. Plné lomené čiary reprezentujú štyri jazdy vopred naplánovaných autobusov (označené indexami). Bodkovaná čierna lomená čiara zobrazuje rezervný autobus.

 `arrival_time(0)`               | `arrival_time(50)`
:-------------------------------:|:------------------------------:
![](overtaking-query1-new.png)   | ![](overtaking-query2-new.png)
## Obmedzenia

* $1 \le L \le 10^9$
* $1 \le N \le 1\,000$
* $0 \le T[i] \le 10^{18}$ (pre každé $i$ také, že $0 \le i \lt N$)
* $1 \le W[i] \le 10^9$ (pre každé $i$ také, že $0 \le i \lt N$)
* $1 \le X \le 10^9$
* $2 \le M \le 1\,000$
* $0 = S[0] \lt S[1] \lt \cdots \lt S[M-1] = L$
* $1 \le Q \le 10^6$
* $0 \le Y \le 10^{18}$

## Podúlohy

1. (09 bodov) $N = 1, Q \le 1\,000$
1. (10 bodov) $M = 2, Q \le 1\,000$
1. (20 bodov) $N, M, Q \le 100$
1. (26 bodov) $Q \le 5\,000$
1. (35 bodov)  bez ďalších obmedzení

## Ukážkový testovač

Ukážkový testovač číta vstup v nasledovnom formáte:

* riadok $1$: $L \; N \; X \; M \; Q$
* riadok $2$: $T[0] \; T[1] \; \ldots \; T[N-1]$
* riadok $3$: $W[0] \; W[1] \; \ldots \; W[N-1]$
* riadok $4$: $S[0] \; S[1] \; \ldots \; S[M-1]$
* riadok $5 + k$ ($0 \le k \lt Q$): $Y$ pre otázku $k$

Ukážkový testovač vypíše výstup v nasledovnom formáte:

* riadok $1 + k$ ($0 \le k \lt Q$): návratová hodnota funkcie `arrival_time` pre otázku $k$
