# Prehitevanje

Od letališča v Budimpešti do hotela Forrás vodi $L$ kilometrov dolga enosmerna enopasovna cesta.

Na dan prihodov na IOI 2023 po tej cesti potuje $N+1$ avtobusov. Avtobusi so oštevilčeni od $0$ do $N$.
Načrtovano je, da avtobus $i$ ($0 \le i \lt N$) zapusti letališče v $T[i]$-ti sekundi; $1$ kilometer je zmožen prevoziti v $W[i]$ sekundah.
Avtobus $N$ je rezervni avtobus; $1$ kilometer je zmožen prevoziti v $X$ sekundah.
Čas $Y$, ob katerem bo zapustil letališče, še ni določen.

Na cesti je prehitevanje prepovedano, vendar pa se avtobusi med seboj lahko prehitijo na **prehitevalnih postajah**.
Na cesti se na različnih mestih nahaja $M$ ($M \gt 1$) prehitevalnih postaj, oštevilčenih od $0$ do $M - 1$.
Prehitevalna postaja $j$ ($0 \le j \lt M$) se nahaja $S[j]$ kilometrov od letališča.
Prehitevalne postaje so razvrščene po naraščajoči oddaljenosti od letališča, torej $S[j] \lt S[j+1]$ za vsak $0 \le j \le M - 2$.
Prva prehitevalna postaja je letališče, zadnja pa hotel: $S[0] = 0$ in $S[M-1] = L$.

Vsak avtobus potuje z največjo hitrostjo, razen če pred seboj dohiti počasnejši avtobus. V tem primeru upočasni in je prisiljen potovati s hitrostjo počasnejšega avtobusa, vse dokler ne pridejo do naslednje prehitevalne postaje.
Tam hitrejši avtobusi prehitijo počasnejše avtobuse.

Formalno, za vsak $i$ in $j$, kjer $0 \le i \le N$ in $0 \le j \lt M$, je čas $t_{i,j}$ (v sekundah), ko avtobus $i$ **pride na** prehitevalno postajo $j$, definiran, kakor sledi.
Naj bo $t_{i,0} = T[i]$ za vsak $0 \le i \lt N$ in naj bo $t_{N,0} = Y$.
Za vsak $j$, kjer $0 \lt j \lt M$:
* Definirajmo **pričakovani čas prihoda** avtobusa $i$ na prehitevalno postajo $j$ (v sekundah). Z $e_{i,j}$ označimo čas, kdaj naj bi avtobus $i$ prišel na prehitevalno postajo $j$, če bi potoval s polno hitrostjo od trenutka, ko prispe na prehitevalno postajo $j-1$. To pomeni:
  - $e_{i,j} = t_{i,j-1} + W[i] \cdot (S[j]-S[j-1])$ za vsak $0 \le i \lt N$, in
  - $e_{N,j} = t_{N,j-1} + X \cdot (S[j]-S[j-1])$.
* Avtobus $i$ prispe na prehitevalno postajo $j$ ob *maksimumu* pričakovanih časov prihodov avtobusa $i$ in vseh ostalih avtobusov, ki so prispeli na postajo $j-1$ pred avtobusom $i$. Formalno naj bo $t_{i,j}$ največji izmed $e_{i,j}$ in vseh $e_{k,j}$, za katere velja $0 \le k \le N$ in  $t_{k,j-1} \lt t_{i,j-1}$

Organizatorji IOI želijo načrtovati rezervni avtobus.
Vaša naloga je odgovoriti na Q vprašanj organizatorjev, ki so naslednje oblike: podan je čas $Y$ (v sekundah), ko naj bi avtobus $N$ zapustil letališče; ob katerem času bo prispel do hotela?

</br>
## Podrobnosti implementacije

Vaša naloga je implementirati naslednjo metodo in funkcijo.

```
void init(int L, int N, int64[] T, int[] W, int X, int M, int[] S)
```

* $L$: dolžina ceste.
* $N$: število ne-rezervnih avtobusov.
* $T$: polje dolžine $N$, ki opisuje čase, ob katerih naj bi ne-rezervni avtobusi zapustili letališče.
* $W$: polje dolžine $N$, ki opisuje najvišje hitrosti ne-rezervnih avtobusov.
* $X$: čas, ki ga potrebuje rezervni avtobus, da prevozi $1$ kilometer.
* $M$: število prehitevalnih postaj.
* $S$: polje dolžine $M$, ki opisuje oddaljenosti prehitevalnih postaj od letališča.
* Ta metoda se pokliče natanko enkrat za vsak testni primer pred klici `arrival_time`.

```
int64 arrival_time(int64 Y)
```

* $Y$: čas, ob katerem naj bi rezervni avtobus zapustil letališče.
* Ta funkcija naj vrne čas, ob katerem bo avtobus $N$ prispel do hotela.
* Ta funkcija se pokliče natanko $Q$-krat.

## Primer

Upoštevajte naslednje zaporedje klicev:

```
init(6, 4, [20, 10, 40, 0], [5, 20, 20, 30], 10, 4, [0, 1, 3, 6])
```

Če ignoriramo avtobus $4$ (ki se še načrtuje), naslednja tabela prikazuje pričakovane in dejanske čase prihodov za ne-rezervne avtobuse na vsaki prehitevalni postaji:

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$

Časi prihodov na postajo $0$ so časi, ob katerih so avtobusi načrtovani za odhod z letališča. To pomeni, da za avtobuse, označene od 0 do 3, velja $t_{i,0} = T[i]$.

Pričakovani in dejanski časi prihodov na prehitevalno postajo 1 so izračunani, kakor sledi:
* Pričakovani časi prihodov na postajo 1:
  - Avtobus 0:  $e_{0,1} = t_{0,0} + W[0] \cdot (S[1]-S[0]) = 20 + 5 \cdot 1 = 25$. 
  - Avtobus 1:  $e_{1,1} = t_{1,0} + W[1] \cdot (S[1]-S[0]) = 10 + 20 \cdot 1 = 30$. 
  - Avtobus 2:  $e_{2,1} = t_{2,0} + W[2] \cdot (S[1]-S[0]) = 40 + 20 \cdot 1 = 60$. 
  - Avtobus 3:  $e_{3,1} = t_{3,0} + W[3] \cdot (S[1]-S[0]) = 0 + 30 \cdot 1 = 30$. 
* Dejanski časi prihodov na postajo 1:
  - Avtobusa 1 in 3 prispeta na postajo 0 prej kot avtobus 0. Zato je vrednost enačbe za avtobus 0 naslednja:  
$t_{0,1} = \max([e_{0,1},e_{1,1},e_{3,1}]) = 30$. 
  - Avtobus 3 prispe na postajo 0 prej kot avtobus 1. Zato je vrednost enačbe za avtobus 1 naslednja:
$t_{1,1} = \max([e_{1,1},e_{3,1}]) = 30$. 
  - Avtobus $0$, avtobus $1$ in avtobus $3$ prispejo na prehitevalno postajo $0$ prej kot avtobus $2$, zato je $t_{2,1} = \max(e_{0,1},e_{1,1},e_{2,1},e_{3,1}) = 60$.
- Noben avtobus ne prispe na postajo $0$ prej kot avtobus številka $3$. Zato je vrednost enačbe za ta avtobus naslednja:
$t_{3,1} = \max([e_{3,1}]) = 30$.

```
arrival_time(0)
```

Avtobus $4$ potrebuje $10$ sekund, da prevozi $1$ kilometer, in je načrtovan, da zapusti letališče v $0$-ti sekundi.
V tem primeru naslednja tabela prikazuje čase prihodov za vsak avtobus.
Edina sprememba glede pričakovanih in dejanskih časov prihoda ne-rezervnih avtobusov je podčrtana.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:-----------------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $\underline{60}$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $0$       || $10$      | $10$      || $30$      | $30$      || $60$      | $60$

Vidimo, da avtobus $4$ prispe do hotela v $60$-ti sekundi.
Zato funkcija vrne $60$.

```
arrival_time(50)
```

Avtobus $4$ je zdaj načrtovan, da letališče zapusti v $50$-ti sekundi.
V tem primeru ni sprememb v časih prihodov za ne-rezervne avtobuse v primerjavi z začetno tabelo.
Časi prihodov so prikazani v naslednji tabeli.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $50$      || $60$      | $60$      || $80$      | $90$      || $120$     | $130$

Avtobus $4$ prehiti počasnejši avtobus $2$ na prehitevalni postaji $1$, saj prispeta ob istem času.
Nato avtobus $4$ ujame avtobus $3$ med postajo $1$ in postajo $2$; kar pomeni, da avtobus $4$ prispe na postajo $2$ v $90$-ti sekundi namesto v $80$-ti.
Po odhodu s postaje $2$ avtobus $4$ ujame avtobus $1$, in se pelje za njim, dokler ne prispeta do hotela.
Avtobus $4$ prispe do hotela v $130$-ti sekundi.
Zato funkcija vrne $130$.

Lahko narišemo čas, ki ga potrebuje vsak avtobus, da prispe na vsako razdaljo od letališča.
os $x$ grafa predstavlja razdaljo od letališča (v kilometrih), os $y$ pa predstavlja čas (v sekundah).
Navpične črtkane črte označujejo položaje prehitevalnih postaj.
Različne polne črte (označene z indeksi avtobusov) predstavljajo štiri načrtovane ne-rezervne avtobuse.
Pikčasta črna črta predstavlja rezervni avtobus.

 `arrival_time(0)`           | `arrival_time(50)`
:---------------------------:|:---------------------------:
![](overtaking-query1.png)   | ![](overtaking-query2.png)

</br>

## Omejitve

* $1 \le L \le 10^9$
* $1 \le N \le 1\,000$
* $0 \le T[i] \le 10^{18}$ (za vsak $i$, tako da $0 \le i \lt N$)
* $1 \le W[i] \le 10^9$ (za vsak $i$, tako da $0 \le i \lt N$)
* $1 \le X \le 10^9$
* $2 \le M \le 1\,000$
* $0 = S[0] \lt S[1] \lt \cdots \lt S[M-1] = L$
* $1 \le Q \le 10^6$
* $0 \le Y \le  10^{18}$

## Podnaloge

1. (9 točk) $N = 1, Q \le 1\,000$
1. (10 točk) $M = 2, Q \le 1\,000$
1. (20 točk) $N, M, Q \le 100$
1. (26 točk) $Q \le 5\,000$
1. (35 točk) Brez dodatnih omejitev.

## Vzorčni ocenjevalnik

Vzorčni ocenjevalnik bere vhod naslednje oblike:

* vrstica $1$: $L \; N \; X \; M \; Q$
* vrstica $2$: $T[0] \; T[1] \; \ldots \; T[N-1]$
* vrstica $3$: $W[0] \; W[1] \; \ldots \; W[N-1]$
* vrstica $4$: $S[0] \; S[1] \; \ldots \; S[M-1]$
* vrstica $5 + k$ ($0 \le k < Q$): Y za vprašanje $k$

Vzorčni ocenjevalnik izpiše vaše odgovore v naslednji obliki:

* vrstica $1 + k$ $(0 \leq k < Q)$: vrednost vračanja `arrival_time` za vprašanje $k$
