# Övertagande

Hotel Forrás ligger en enkel bilresa från Budapests flygplats. Längden på vägen är $L$ kilometer.

Under IOI 2023-evenemanget kommer $N+1$ bussar att köra på denna rutt. Bussar är numrerade från $0$ till $N$.
Buss $i$ ( $0 \le i \lt N$ ) är planerad att lämna flygplatsen vid $T[i]$ sekunden av evenemanget och kan resa $1$ kilometer på $W[i]$ sekunder.
Buss $N$ är en alternativ buss som kan resa 1$ kilometer på $X$ sekunder.
Det är ännu inte bestämt när $Y$ ska lämna flygplatsen.

Omkörning på väg är generellt förbjudet, men bussar får köra om varandra vid **manövreringsstationer**.
Det finns $M$ sorteringsstationer på gatan, numrerade från $0$ till $M-1$.
Klassificeringsstationen $j$ ( $0 \le j \lt M$ ) ligger $S[j]$ kilometer från flygplatsen längs vägen.
Den första sorteringsstationen är flygplatsen, och den sista är hotellet, d.v.s. $S[0] = 0$ och $S[M-1] = L$.

Varje buss färdas med maximal hastighet om den inte kör om en långsammare buss framför sig på vägen. I det här fallet bildar de en grupp och måste färdas med samma reducerade hastighet tills de når nästa sorteringsstation.
Formellt, för varje $i$ och $j$ med $0 \le i \le N$ och $0 \le j \lt M$, tillämpas tiden $t_{i,j}$ (i sekunder) om buss $i $ * *anländer* * sorteringsstation $j$ definieras enligt följande.
Om $j = 0$, låt $t_{i,0} = T[i]$ för varje $i \lt N$ och $t_{N,0} = Y$.
Annars, för varje $j$ med $0 \lt j \lt M$:
* Förutsäg *beräknad ankomsttid* för buss $i$ vid rangeringsstationen $j$ som den tid det skulle ta buss $i$ att anlända till rangeringsstationen $j$ om den var full från ankomst till rangeringsstationen $ Velocity. på vägen skulle vara j-1$. Det betyder att lämna
      - $e_{i,j} = t_{i,j-1} + W[i] \cdot (S[j]-S[j-1])$ för varje $i \lt N$ och
      - $e_{N,j} = t_{N,j-1} + X \cdot (S[j]-S[j-1])$.
* Buss $i$ anländer till sorteringsstationen $j$ vid *maximal* ankomsttid för buss $i$ och en annan buss som anlände tidigare än buss $i$ vid stationen $j-1$. Formellt, låt $t_{i,j}$ vara den största $e_{i,j}$ och varje $e_{k,j}$ som $0 \le k \le N$ och $t_{k,j - 1 } \ en t_{i,j-1}$.

IOI-arrangörer vill organisera en alternativ transportbuss.
Din uppgift är att svara på planerarens $Q$ frågor, vilka är: Om buss $N$ måste lämna flygplatsen, givet tid $Y$ (i sekunder), vid vilken tid kommer den att anlända till flygplatsen. hotell?

## Implementation Details

Your task is to implement the following procedures.

```
void init(int L, int N, int64[] T, int[] W, int X, int M, int[] S)
```

* $L$: the length of the road.
* $N$: the number of scheduled transfer buses.
* $T$: an array of length $N$ describing the times at which buses $0,\ldots,N-1$ are scheduled to leave from the airport.
* $W$: an array of length $N$ describing the maximum speeds of buses $0,\ldots,N-1$.
* $X$: the time it takes for the reserve bus to travel $1$ kilometre.
* $M$: the number of sorting stations.
* $S$: an array of length $M$ describing the distances of the sorting stations from the airport.
* This procedure is called exactly once for each test case, before any calls to `arrival_time`.

```
int64 arrival_time(int64 Y)
```

* $Y$: the time at which the reserve bus is supposed to leave from the airport.
* This procedure should return the time at which bus $N$ would arrive at the hotel.
* This procedure is called exactly $Q$ times.

## Example

Consider the following sequence of calls:

```
init(6, 4, [20, 10, 40, 0], [5, 20, 20, 30], 10, 4, [0, 1, 3, 6])
```

Ignoring bus $4$ (that has not yet been scheduled), the following table shows the expected and actual times of arrivals for buses $0,1,2$ and $3$ at each sorting station:

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$

The times of arrivals at station $0$ are the times at which buses are scheduled to leave the airport. That is, $t_{i,0} = T[i]$ for $i=0,1,2$ and $3$.

The expected and actual times of arrivals at sorting station $1$ are computed as follows:
* The expected times of arrivals at station $1$:
  - Bus $0$: $e_{0,1} = t_{0,0} + W[0] \cdot (S[1]-S[0]) = 20 + 5 \cdot 1 = 25$.
  - Bus $1$: $e_{1,1} = t_{1,0} + W[1] \cdot (S[1]-S[0]) = 10 + 20 \cdot 1 = 30$.
  - Bus $2$: $e_{2,1} = t_{2,0} + W[2] \cdot (S[1]-S[0]) = 40 + 20 \cdot 1 = 60$.
  - Bus $3$: $e_{3,1} = t_{3,0} + W[3] \cdot (S[1]-S[0]) = 0 + 30 \cdot 1 = 30$.
* The times of arrivals at station $1$:
  - Buses $1$ and $3$ arrive at station $0$ earlier than bus $0$, so $t_{0,1} = \max(e_{0,1},e_{1,1},e_{3,1}) = 30$.
  - Bus $3$ arrives at station $0$ earlier than bus $1$, so $t_{1,1} = \max(e_{1,1},e_{3,1}) = 30$.
  - Bus $0$, bus $1$ and bus $3$ arrives at sorting station $0$ earlier than bus $2$, so $t_{2,1} = \max(e_{0,1},e_{1,1},e_{2,1},e_{3,1}) = 60$.
  - No bus arrives at station $0$ earlier than bus $3$, so $t_{3,1} = \max(e_{3,1}) = 30$.

We can plot the distances of buses from the airport (in kilometres) against time (in seconds) as shown in the figure below.
Vertical dashed lines mark the positions of the sorting stations. 
Different broken lines (accompanied by the bus indices) represent the four scheduled transfer buses.

![Cars](overtaking-buses.png "400")

```
arrival_time(0)
```

Bus $4$ takes $10$ seconds to travel $1$ kilometre and is now scheduled to leave the airport at the $0$-th second.
In this case, the following table shows the times of arrivals for each bus.
Changes compared to the initial table are underlined.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:-----------------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $\underline{60}$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $0$       || $10$      | $10$      || $30$      | $30$      || $60$      | $60$

We see that bus $4$ arrives at the hotel at the $60$-th second.
Thus, the procedure should return $60$.

```
arrival_time(50)
```

Bus $4$ is now scheduled to leave the airport at the $50$-th second.
In this case, there are no changes in the times of arrivals for buses $0$, $1$, $2$ and $3$.
The times of arrivals are shown in the following table.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $50$      || $60$      | $60$      || $80$      | $90$      || $120$     | $130$

Bus $4$ overtakes the slower bus $2$ at sorting station $1$ as they arrive at the same time.
Next, bus $4$ gets bunched with bus $3$ between station $1$ and station $2$, making bus $4$ arrive at station $2$ at the $90$-th second instead of the $80$-th.
After leaving station $2$, bus $4$ gets bunched with bus $1$ up until they arrive at the hotel.
Bus $4$ arrives at the hotel at the $130$-th second.
Thus, the procedure should return $130$.

In the following figures, we plot the distances of the buses from the airport against time. The dotted black line represents the reserve bus.

 `arrival_time(0)`           | `arrival_time(50)`
:---------------------------:|:---------------------------:
![](overtaking-query1.png)   | ![](overtaking-query2.png)

## Constraints

* $1 \le L \le 10^9$
* $1 \le N \le 1\,000$
* $0 \le T[i] \le 10^{18}$ (for each $i$ such that $0 \le i \lt N$)
* $1 \le W[i] \le 10^9$ (for each $i$ such that $0 \le i \lt N$)
* $1 \le X \le 10^9$
* $2 \le M \le 1\,000$
* $0 = S[0] \lt S[1] \lt \cdots \lt S[M-1] = L$
* $1 \le Q \le 10^6$
* $0 \le Y \le  10^{18}$

## Subtasks

1. (9 points) $N = 1, Q \le 1\,000$
1. (10 points) $M = 2, Q \le 1\,000$
1. (20 points) $N, M, Q \le 100$
1. (26 points) $Q \le 5\,000$
1. (35 points) No additional constraints.

## Sample Grader

The sample grader reads the input in the following format:

* line $1$: $L \; N \; X \; M \; Q$
* line $2$: $T[0] \; T[1] \; \ldots \; T[N-1]$
* line $3$: $W[0] \; W[1] \; \ldots \; W[N-1]$
* line $4$: $S[0] \; S[1] \; \ldots \; S[M-1]$
* line $5 + k$ ($0 \le k \lt Q$): $Y$ for question $k$

The sample grader prints your answers in the following format:

* line $1 + k$ ($0 \le k \lt Q$): the return value of `arrival_time` for question $k$
