# Sollama

Budapeşte Havaalanı'ndan Hotel Forrás'a tek şeritli, tek yönlü bir yol bulunmaktadır. Bu yol $L$ kilometre uzunluğundadır.

IOI 2023 etkinliğinde $N+1$ tane transfer otobüsü bu yoldan geçer. Otobüsler $0$ ile $N$ arasında numaralandırılmıştır.
$i$ otobüsünün ($0 \le i \lt N$) , etkinliğin $T[i]$'inci saniyesinde havaalanından ayrılması planlanıyor ve $W[i]$ saniyede $1$ kilometre yol kat edebilir.
$N$ otobüsü, $1$ kilometreyi $X$ saniyede gidebilen bir yedek otobüstür.
Havaalanından ayrılacağı zamana, $Y$, henüz karar verilmedi.

Yolda sollamaya genel olarak izin verilmiyor ancak **sıralama istasyonlarında** otobüslerin birbirini geçmesine izin veriliyor.
Yolda $0$'dan $M - 1$'e kadar numaralandırılmış yolun farklı pozisyonlarında olan $M$ sıralama istasyonu vardır.
$j$ sıralama istasyonu ($0 \le j \lt M$), yolda havaalanından $S[j]$ kilometre uzakta bulunur.
Sıralama istasyonları havaalanından artan mesafeye göre sıralanır, yani her $0 \le j \le M - 2$ için $S[j] \lt S[j+1]$.
İlk sıralama istasyonu havaalanında ve sonuncusu ise oteldir, yani $S[0] = 0$ ve $S[M-1] = L$.

Her otobüs, yolda kendisinden daha yavaş ilerleyen bir otobüse yetişemediği sürece maksimum hızda hareket eder; bu durumda otobüsler birlikte bir sonraki sıralama istasyonuna ulaşana kadar yavaş giden otobüsün hızında seyahat eder. Sıralama istasyonunda, hızlı otobüs yavaş olanı geçer. 

Biçimsel olarak, $0 \le i \le N$ ve $0 \le j \lt M$ olacak şekilde her $i$ ve $j$ için, saniye cinsinden $t_{i,j}$ zamanı (yani $i$ otobüsü $j$ istasyonuna **vardığındaki** zaman) şu şekilde tanımlanır. 
Her $i \lt N$ için $t_{i,0} = T[i]$ olsun ve $t_{N,0} = Y$ olsun. 
Aksi halde, $0 \lt j \lt M$ olacak şekilde her $j$ için:

$0 \lt j \lt M$ olacak şekilde her $j$ için:
* $i$ otobüsünün $j$ sıralama istasyonuna saniye olarak **beklenen varış zamanını** ($e_{i,j}$ ile gösterilir) şu şekilde tanımlayın: Eğer $i$  otobüsü $j-1$ sıralama istasyonuna vardıktan sonra tam hızla yoluna devam edebilseydi, bu otobüsünün, $j$ sıralama istasyonuna varabilme zamanı olarak tanımlayın.Yani:
   - $e_{i,j} = t_{i,j-1} + W[i] \cdot (S[j]-S[j-1])$ her $0 \le i \lt N$ için ve
   - $e_{N,j} = t_{N,j-1} + X \cdot (S[j]-S[j-1])$.
* $i$ otobüsü, sıralama istasyonu $j$'ye $i$ 'nin beklenen varış zamanlarının *en büyüğünde* ve istasyon $j-1$'e otobüs $i$'den önce varmış olan herhangi bir otobüsün beklenen varış zamanlarının *en büyüğünde* gelir. Biçimsel olarak ifade edecek olursak, $t_{i,j}$, $e_{i,j}$'nin ve $0 \le k \le N$ aralığında olan ve $t_{k,j-1} \lt t_{i,j-1}$ koşulunu sağlayan her $e_{k,j}$'nin en büyüğüdür.

IOI organizatörleri yedek otobüsü ($N$ numaralı otobüs) planlamak istiyor.
Göreviniz, organizatörlerin aşağıdaki formda olan $Q$ tane sorusunu yanıtlamaktır: $Y$ (saniye cinsinden) süresi verildiğinde, yani yedek otobüsünün havaalanından ayrılması gereken zaman verildiğinde, o otobüs otele saat kaçta varır?

## Kodlama Detayları

Göreviniz aşağıdaki prosedürleri kodlamakdır.

```
void init(int L, int N, int64[] T, int[] W, int X, int M, int[] S)
```

* $L$: Yolun uzunluğu.
* $N$: Yedek olmayan otobüslerin sayısı.
* $T$: Yedek olmayan otobüslerin havaalanından ayrılmasının planlandığı saatleri tanımlayan $N$ uzunluğundaki dizi.
* $W$: Yedek olmayan otobüslerin maksimum hızlarını tanımlayan $N$ uzunluğundaki dizi.
* $X$: Yedek otobüsün $1$ kilometre yol alması için gereken süre.
* $M$: Sıralama istasyonlarının sayısı.
* $S$: Sıralama istasyonlarının havaalanına olan mesafelerini tanımlayan $M$ uzunluğundaki bir dizi.
* Bu prosedür, her test senaryosu için, herhangi bir `arrival_time` çağrısından önce tam olarak bir kez çağrılır.

```
int64 arrival_time(int64 Y)
```

* $Y$: Yedek otobüsün ($N$ numaralı otobüsün) havaalanından ayrılması gereken saat.
* Bu prosedür, yedek otobüsün otele varacağı zamanı dönmelidir.
* Bu prosedür tam olarak $Q$ kere çağrılmalıdır.

## Örnek

Aşağıdaki çağrı sırasını göz önünde bulundurun:

```
init(6, 4, [20, 10, 40, 0], [5, 20, 20, 30], 10, 4, [0, 1, 3, 6])
```

Otobüs $4$'ü (henüz planlanmamış olan) göz ardı ederek aşağıdaki tablo, her bir sıralama istasyonuna rezerve edilmeyen otobüslerin beklenen ve gerçek varış saatlerini göstermektedir:

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$

$0$ istasyonuna varış saatleri, otobüslerin havaalanından ayrılmasının planlandığı saatlerdir. Yani, $0 \le i \le 3$. için $t_{i,0} = T[i]$.

$1$ sıralama istasyonunda beklenen ve gerçek varış süreleri aşağıdaki şekilde hesaplanır:
* $1$ istasyonuna beklenen varış saatleri:
  - Otobüs $0$: $e_{0,1} = t_{0,0} + W[0] \cdot (S[1]-S[0]) = 20 + 5 \cdot 1 = 25$.
  - Otobüs $1$: $e_{1,1} = t_{1,0} + W[1] \cdot (S[1]-S[0]) = 10 + 20 \cdot 1 = 30$.
  - Otobüs $2$: $e_{2,1} = t_{2,0} + W[2] \cdot (S[1]-S[0]) = 40 + 20 \cdot 1 = 60$.
  - Otobüs $3$: $e_{3,1} = t_{3,0} + W[3] \cdot (S[1]-S[0]) = 0 + 30 \cdot 1 = 30$.
* $1$ istasyonuna varış saatleri:
  - $1$ ve $3$ otobüsleri $0$ istasyonuna $0$ otobüsten daha erken varır, dolayısıyla $t_{0,1} = \max([e_{0,1},e_{1,1},e_{3,1}]) = 30$.
  - $3$ otobüsü $0$ istasyonuna $1$ otobüsünden daha erken varır, yani $t_{1,1} = \max([e_{1,1},e_{3,1}]) = 30$.
  - Otobüs $0$, otobüs $1$ ve otobüs $3$, sıralama istasyonu $0$'a otobüs $2$'den daha erken varır, yani $t_{2,1} = \max([e_{0,1},e_{1,1},e_{2,1},e_{3,1}]) = 60$.
  - Hiçbir otobüs $0$ istasyonuna $3$ numaralı otobüsten daha erken varmaz, yani $t_{3,1} = \max([e_{3,1}]) = 30$.


```
arrival_time(0)
```

$4$ otobüsün $1$ kilometrelik mesafeyi kat etmesi $10$ saniye sürüyor ve şimdi havaalanından $0$'ıncı saniyede ayrılması planlanıyor.
Bu durumda, aşağıdaki tabloda her otobüsün varış saatini göstermektedir.
Rezervasyon yapılmayan otobüslerin beklenen ve gerçek varış saatlerine ilişkin tek değişikliğin altı çizilmiştir.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:-----------------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $\underline{60}$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $0$       || $10$      | $10$      || $30$      | $30$      || $60$      | $60$

$4$ otobüsün otele $60$'ıncı saniyede geldiğini görüyoruz.
Bu nedenle, prosedür $60$ değerini dönmelidir.

```
arrival_time(50)
```

$4$ numaralı otobüsün artık $50$'ıncı saniyede havaalanından ayrılması planlanıyor.
Bu durumda, rezerve edilmeyen otobüslerin varış saatlerinde başlangıç tablosuna göre bir değişiklik görülmemektedir.
Varış saatleri aşağıdaki tabloda gösterilmektedir.

$i$ || $t_{i,0}$ || $e_{i,1}$ | $t_{i,1}$ || $e_{i,2}$ | $t_{i,2}$ || $e_{i,3}$ | $t_{i,3}$
:--:||:---------:||:---------:|:---------:||:---------:|:---------:||:---------:|:---------:
$0$ || $20$      || $25$      | $30$      || $40$      | $40$      || $55$      | $55$
$1$ || $10$      || $30$      | $30$      || $70$      | $70$      || $130$     | $130$
$2$ || $40$      || $60$      | $60$      || $100$     | $100$     || $160$     | $180$
$3$ || $0$       || $30$      | $30$      || $90$      | $90$      || $180$     | $180$
$4$ || $50$      || $60$      | $60$      || $80$      | $90$      || $120$     | $130$

$4$ numaralı otobüs, aynı anda vardıklarından dolayı $1$ sıralama istasyonunda, kendinden daha yavaş olan $2$ numaralı otobüsü solluyor.
Daha sonra, $4$ otobüsü, $1$ istasyonu ile $2$ istasyonu arasında $3$ numaralı otobüsle bir araya gelir ve bu $4$ otobüsünün $2$ istasyonuna $80$'inci saniye yerine $90$'ıncı saniyede varmasını sağlar.
$2$ istasyonundan ayrıldıktan sonra, $4$ otobüsü, otele varana kadar $1$ numaralı otobüsle birlikte seyahat eder.
$4$ otobüsü otele $130$'ıncı saniyede varır.
Bu nedenle, prosedür $130$'u dönmelidir.

Her otobüsün havaalanından her mesafeye varması için geçen süreyi şekil üzerinde gösterebiliriz.
Grafiğin x ekseni havaalanına olan mesafeyi (kilometre cinsinden), y ekseni ise zamanı (saniye cinsinden) temsil etmektedir.
Dikey kesikli çizgiler, sıralama istasyonlarının konumlarını gösterir.
Farklı düz çizgiler (otobüs indisleri eşliğinde), yedek olmayan dört otobüsü temsil eder.
Noktalı siyah çizgi yedek otobüsü temsil eder.


 `arrival_time(0)`           | `arrival_time(50)`
:-------------------------------:|:------------------------------:
![](overtaking-query1-new.png)   | ![](overtaking-query2-new.png)

## Kısıtlar

* $1 \le L \le 10^9$
* $1 \le N \le 1\,000$
* $0 \le T[i] \le 10^{18}$ ($0 \le i \lt N$ olacak şekilde her $i$ için)
* $1 \le W[i] \le 10^9$ ($0 \le i \lt N$ olacak şekilde her $i$ için)
* $1 \le X \le 10^9$
* $2 \le M \le 1\,000$
* $0 = S[0] \lt S[1] \lt \cdots \lt S[M-1] = L$
* $1 \le Q \le 10^6$
* $0 \le Y \le  10^{18}$

## Altgörevler

1. (9 puan) $N = 1, Q \le 1\,000$
1. (10 puan) $M = 2, Q \le 1\,000$
1. (20 puan) $N, M, Q \le 100$
1. (26 puan) $Q \le 5\,000$
1. (35 puan) Ek kısıt yoktur.

## Örnek Değerlendirici (Sample Grader)

Örnek değerlendirici girdiyi aşağıdaki formatta okur:

* satır $1$: $L \; N \; X \; M \; Q$
* satır $2$: $T[0] \; T[1] \; \ldots \; T[N-1]$
* satır $3$: $W[0] \; W[1] \; \ldots \; W[N-1]$
* satır $4$: $S[0] \; S[1] \; \ldots \; S[M-1]$
* satır $5 + k$ ($0 \le k \lt Q$): $Y$,  $k$ sorusu için

Örnek değerlendirici cevaplarınızı aşağıdaki formatta yazar:

* satır $1 + k$ ($0 \le k \lt Q$): $k$ sorusu için `arrival_time` 'ın dönen değeri
